// $Id: cgi.C,v 1.11 2003/10/21 12:37:52 dvermeir Exp $
#include <stdexcept>
#include <fstream>
#include <string>

#include <dvcgi/cgi.h>
#include <dvutil/debug.h>
#include <dvcgi/wwwdecode.h>
#include <dvnet/inetaddress.h>
#include "cgi.h"

Dv::Ticket::Cgi::Cgi(const std::string& name, 
  const std::string& login_url, const std::string& default_language, bool ssl_only,
  const std::string& type, bool debug) throw (): Dv::Cgi::Cgi(name, type.c_str(), debug), 
  ticket_(), is_ssl_(false), language_(default_language) {

static const std::string HTTPS("https://");

try {
  if ( header().debug() )
    log() << "Dv::Ticket::Cgi ENTER:\n" << Dv::Util::Debug<Dv::Ticket::Cgi>(*this) << std::endl;


  is_ssl_ = ( env()("SSL_PROTOCOL") || 
              ( env()("HTTPS") ?  *(env()("HTTPS")) == "on" : false)
	    );

  ///////////////////////////////////////////////////////////////////
  // Redirect if ssl is required and this is not an SSL connection
  if ( ssl_only && ! is_ssl() ) {
    const std::string https_here( HTTPS + this_host() + here() );
    if ( header().debug() ) 
      log() << "### Dv::Ticket::Cgi EXIT: redirect to: " << https_here.c_str() << std::endl;
    header().nocache().location(https_here);
    *this << std::endl;
    exit (0);
    }

  ///////////////////////////////////////////////////////////////////
  // Initialize ticket information

  Dv::Ticket::Ticket::SERIAL serial(0);
  std::string ticket_server_host;
  int ticket_server_port(0);

  ///////////////////////////////////////////////////////////////////
  // If ticket is in props(), it overrides any cookies, we set the
  // cookies and relocate to ourselves.

  if ( props().find("ticket") ) {

    if ( header().debug() )
      log() << "### Dv::Ticket::Cgi: ticket in props found" << std::endl;

    props() >> "ticket" >> serial
            >> "ticket-server-host" >> ticket_server_host
            >> "ticket-server-port" >> ticket_server_port
	    >> "language" >> language_;

    props().erase("ticket")
           .erase("ticket-server-host")
           .erase("ticket-server-port")
           .erase("language");
    
    std::string clean_url( url(props()) );

    header().cookie("language", language_)
            .cookie("ticket-server-host", ticket_server_host)
            .cookie("ticket-server-port", Dv::Util::tostring(ticket_server_port))
            .cookie("ticket", Dv::Util::tostring(serial) )
	    .content_type("text/html")
    	    .refresh(1, clean_url); // note: location() does not set cookies immediately!

    if ( header().debug() )
      log() << "### Dv::Ticket::Cgi EXIT: redirect to clean url: " << clean_url.c_str() << std::endl;
    *this << 
        "<html>"
        "<head><title>login ok</title></head>"
        "<body bgcolor=\"#ffffff\">"
        "<h2>login ok</h2>"
      << "<p>Automatic redirection or click "
         "<a href=\"" << clean_url.c_str() << "\">here</a>."
         "</body>" 
         "</html>"
      << std::endl;
    exit(0); 
    }

  ///////////////////////////////////////////////////////////////////
  // If ticket is in cookies(), we just try to validate it. This is
  // the only possibility for a succesful return from the constructor
  // (without exiting the program).

  if ( cookies().find("ticket") ) {
    if ( header().debug() )
      log() << "### Dv::Ticket::Cgi: ticket in cookies found" << std::endl;

    cookies() >> "ticket" >> serial
              >> "ticket-server-host" >> ticket_server_host
              >> "ticket-server-port" >> ticket_server_port
	      >> "language" >> language_;

    ticket_ =  Dv::Ticket::Ticket::validate(
      ticket_server_host, ticket_server_port, client_host(), serial);

    if ( ticket_ ) {
      if ( header().debug() )
        log() << "### Dv::Ticket::Cgi RETURN: ticket = " << ticket_->serial() << std::endl; 
      return; //// the only succesful exit
      }
    else
      if ( header().debug() )
        log() << "### Dv::Ticket::Cgi: cookies ticket did not validate" << std::endl;
    }

  ///////////////////////////////////////////////////////////////////
  // No valid ticket, delegate to login program.
  if ( header().debug() )
    log() << "### Dv::Ticket::Cgi: no valid ticket available" << std::endl;

  // assert (! ticket_ )

  // Url to where the login program should redirect, after a succesful login.
  Dv::Util::Props login_props;
  login_props["url"] = (HTTPS + this_host() +  here());
  std::string full_login_url(Dv::Cgi::Cgi::url(HTTPS + login_url, login_props));

  if ( header().debug() )
    log() << "### Dv::Ticket::Cgi EXIT: redirect to login url: " << full_login_url.c_str() << std::endl;

  header().location(full_login_url);
  *this << std::endl;
  exit(0);
  }
catch (std::exception& e) {
  log() << e.what() << std::endl;
  header().content_type("text/plain");
  *this << e.what() << std::endl;
  exit(1);
  }
}

Dv::Ticket::Cgi::~Cgi() {
}


