// $Id: login-cgi.C,v 1.10 2003/10/07 12:47:39 dvermeir Exp $

#include <stdexcept>
#include <fstream>
#include <sstream>

#include <dvutil/file.h>
#include <dvutil/debug.h>
#include <dvcgi/cgi.h>
#include <dvxml/xml.h>

#include "ticket.h"

int
main(int argc, char* argv[]) {
static const std::string HTTPS("https://");

static const char* NAME("login.cgi");
static const char* URL("url");
static const char* USERNAME("username");
static const char* PASSWORD("password");
static const char* LANGUAGE("language");
static const char* CLIENTHOST("clienthost");
static const char* ACTION("action");

Dv::Cgi::Cgi cgi(NAME, "text/html", true);

try {
  if (argc != 2)
    throw std::runtime_error("usage: login.cgi config-file");

  cgi.log() << Dv::Util::Debug<Dv::Cgi::Cgi>(cgi) << std::endl;

  /////////////////////////////////////////////////////////////////////////////////
  // Obtain configuration parameters and language info, if any, from cookies.

  std::string config_fn(argv[1]);
  Dv::Util::File config_f(config_fn);
  if (! config_f.exists() )
    throw std::runtime_error(config_fn + ": cannot open");
  
  Dv::Xml::Document config(config_f);
  
  // The ticket server should be on the same host (localhost) as the
  // login program, so we only need to know its root and get the host
  // from the cgi.env().
  int port = (config.root()/"server"/"host"/"port");
  std::string server_name(cgi.this_host_name());

  // Determine the language to be used.  The default language can be
  // overruled by the application calling this cgi script.
  std::string language = (config.root()/"default-language");
  if ( cgi.props()(LANGUAGE) )
    language = cgi.props(LANGUAGE);

  // Default url to which we will redirect after login, this should
  // really not be used, it is here as a safety measure (in case the
  // application did not define it).  Thus, the default url should be
  // overruled by the application calling this cgi script.
  std::string url = (config.root()/"default-url");
  if ( cgi.props()(URL) )
    url = cgi.props(URL);


  /////////////////////////////////////////////////////////////////////////////////
  // Is this an SSL request? If not, redirect to the https version.

  bool is_ssl = ( cgi.env()("SSL_PROTOCOL") || 
                  ( cgi.env()("HTTPS") ?  *(cgi.env()("HTTPS")) == "on" : false)
	        );

  if (! is_ssl ) {
    cgi.log() << "#### relocate to https://" << std::endl;
    cgi.header().nocache().location( HTTPS + cgi.this_host() + cgi.here() );
    cgi << std::endl;
    return 0;
    }
  
  /////////////////////////////////////////////////////////////////////////////////
  // Initial page, we assume this is the initial "call" if there is no
  // "username" formdata field.

  if (! cgi.props().find(USERNAME) ) { 

    // Find html form filename to display
    Dv::Xml::Node::Ref html(config.root()/"html-file");
    while (html("language")!= language)
      ++html;
    std::string html_fn = html;

    // The actual form contains strings like %{clienthost} which
    // should be substituted. We prepare a Props object that
    // contains definitions for CLIENTHOST, ACTION, LANGUAGE, URL
    Dv::Util::Props out;
    out.add(CLIENTHOST, cgi.client_host())
       .add(ACTION, cgi.here_base())
       .add(LANGUAGE, language)
       .add(URL, url);

    // Grab the form html file, substitute stuff and send it to
    // the browser.
    std::ifstream ifs(html_fn.c_str());
    if (! ifs)
      throw std::runtime_error(html_fn + ": cannot open");
    std::ostringstream oss;
    out.substitute(ifs, oss);
    cgi.log() << "#### show login form" << std::endl;
    cgi << oss.str() << std::endl;
    return 0;
    }

  /////////////////////////////////////////////////////////////////////////////////
  // Final page, because it contains a "username" formdata field.

  // TODO: Verify that this request was generated by ourselves, to
  // prevent other programs getting passwd etc.

  std::string username;
  std::string passwd;
  std::string client_host; // enduser host, set by 1st screen

  cgi.props() >> USERNAME >> username 
              >> PASSWORD >> passwd 
	      >> URL >> url 
	      >> LANGUAGE >> language
	      >> CLIENTHOST >> client_host;

  // buy a ticket on behalf of the client_host host
  Dv::Ticket::Ticket::Ref ticket(
    Dv::Ticket::Ticket::buy(client_host, server_name, port, username, passwd) );

  if (ticket) {
    ///////////////////////////////////////////////////////////////////////////////
    // Final page and a valid ticket was obtained.
    Dv::Util::Props ticket_props;
    ticket_props["ticket"] =  ticket->serial();
    ticket_props["language"] = language;
    ticket_props["ticket-server-host"] = server_name;
    ticket_props["ticket-server-port"]= port;

    cgi.log() << "#### got ticket" << std::endl;
    cgi.header().content_type("text/plain")
                .location(Dv::Cgi::Cgi::url(url, ticket_props));
    cgi << std::endl;
    return 0;
    }
  else {
    ///////////////////////////////////////////////////////////////////////////////
    // Final page and no valid ticket was obtained.
    cgi.log() << "#### failed to get ticket" << std::endl;
    cgi.header().nocache().location(cgi.here());
    }
  }
catch (std::exception& e) {
  cgi.header().content_type("text/plain");
  cgi << e.what() << std::endl;
  return 1;
  }
return 0;
}

